# Copyright 2026 Datastrato, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Shared utilities for the ADP SDK."""

from .exceptions import (
    ADPError,
    ExecutionFailedError,
    InternalError,
    InvalidParamsError,
    InvalidRequestError,
    MethodNotFoundError,
    ParseError,
    ResourceNotFoundError,
    UnauthorizedError,
    ValidationFailedError,
    get_exception_for_error_code,
)
from .message import (
    create_error_from_exception,
    create_error_response,
    create_result_response,
    deserialize_json,
    deserialize_request,
    serialize_message,
)

__all__ = [
    # Exceptions
    "ADPError",
    "ExecutionFailedError",
    "InternalError",
    "InvalidParamsError",
    "InvalidRequestError",
    "MethodNotFoundError",
    "ParseError",
    "ResourceNotFoundError",
    "UnauthorizedError",
    "ValidationFailedError",
    "get_exception_for_error_code",
    # Message utilities
    "create_error_from_exception",
    "create_error_response",
    "create_result_response",
    "deserialize_json",
    "deserialize_request",
    "serialize_message",
]
