# Copyright 2026 Datastrato, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
ADP Types - Pydantic models for the Agentic Data Protocol.

This module re-exports all types for convenient access.
"""

from .common import (
    Capabilities,
    ClientCapabilities,
    ConsistencyLevel,
    ExecutionMetadata,
    Field,
    FieldDefinition,
    FieldMetadata,
    FieldType,
    IdentityPredicate,
    Implementation,
    IntentClass,
    IssueSeverity,
    LogicOperator,
    MutableCapability,
    Predicate,
    PredicateCapability,
    PredicateExpression,
    PredicateGroup,
    PredicateOperator,
    PredicateUsage,
    PredicateValue,
    ProjectionCapability,
    Resource,
    ResourceId,
    ServerCapabilities,
    SimilarValue,
    SortOrder,
    TraceId,
    UsageContract,
    ValidationIssue,
    ValidationIssueCode,
    VectorMetadata,
)
from .intents import (
    IngestIntent,
    Intent,
    LookupIntent,
    QueryIntent,
    ReviseIntent,
)
from .jsonrpc import (
    EXECUTION_FAILED,
    INTERNAL_ERROR,
    INVALID_PARAMS,
    INVALID_REQUEST,
    JSONRPC_VERSION,
    LATEST_PROTOCOL_VERSION,
    METHOD_NOT_FOUND,
    PARSE_ERROR,
    RESOURCE_NOT_FOUND,
    SUPPORTED_PROTOCOL_VERSIONS,
    UNAUTHORIZED,
    VALIDATION_FAILED,
    Cursor,
    EmptyResult,
    JSONRPCError,
    JSONRPCErrorResponse,
    JSONRPCMessage,
    JSONRPCRequest,
    JSONRPCResponse,
    JSONRPCResultResponse,
    ProgressToken,
    RequestId,
    RequestMeta,
    RequestParams,
    Result,
)
from .requests import (
    DescribeRequestParams,
    DiscoverFilter,
    DiscoverRequestParams,
    ExecuteRequestParams,
    InitializeRequestParams,
    PaginatedRequestParams,
    ValidateRequestParams,
)
from .responses import (
    DescribeResult,
    DiscoverResult,
    ExecuteResult,
    InitializeResult,
    PaginatedResult,
    ValidateResult,
)

__all__ = [
    # JSON-RPC
    "JSONRPC_VERSION",
    "LATEST_PROTOCOL_VERSION",
    "SUPPORTED_PROTOCOL_VERSIONS",
    "RequestId",
    "ProgressToken",
    "Cursor",
    "RequestMeta",
    "RequestParams",
    "Result",
    "EmptyResult",
    "JSONRPCError",
    "JSONRPCRequest",
    "JSONRPCResultResponse",
    "JSONRPCErrorResponse",
    "JSONRPCResponse",
    "JSONRPCMessage",
    # Error codes
    "PARSE_ERROR",
    "INVALID_REQUEST",
    "METHOD_NOT_FOUND",
    "INVALID_PARAMS",
    "INTERNAL_ERROR",
    "RESOURCE_NOT_FOUND",
    "VALIDATION_FAILED",
    "UNAUTHORIZED",
    "EXECUTION_FAILED",
    # Common types
    "ResourceId",
    "TraceId",
    "IntentClass",
    "PredicateOperator",
    "LogicOperator",
    "IssueSeverity",
    "ConsistencyLevel",
    "FieldType",
    "PredicateUsage",
    "ValidationIssueCode",
    "SimilarValue",
    "PredicateValue",
    "Predicate",
    "IdentityPredicate",
    "PredicateGroup",
    "PredicateExpression",
    "Implementation",
    "ClientCapabilities",
    "ServerCapabilities",
    "Resource",
    "VectorMetadata",
    "FieldMetadata",
    "Field",
    "FieldDefinition",
    "PredicateCapability",
    "ProjectionCapability",
    "MutableCapability",
    "Capabilities",
    "UsageContract",
    "SortOrder",
    "ValidationIssue",
    "ExecutionMetadata",
    # Intents
    "LookupIntent",
    "QueryIntent",
    "IngestIntent",
    "ReviseIntent",
    "Intent",
    # Requests
    "PaginatedRequestParams",
    "InitializeRequestParams",
    "DiscoverFilter",
    "DiscoverRequestParams",
    "DescribeRequestParams",
    "ValidateRequestParams",
    "ExecuteRequestParams",
    # Responses
    "PaginatedResult",
    "InitializeResult",
    "DiscoverResult",
    "DescribeResult",
    "ValidateResult",
    "ExecuteResult",
]
